% Written by ngelo Cardoso <angelo.cardoso@ist.utl.pt>
% 18 July 2012

% The detail of the algorithm is described in the following paper:
% ngelo Cardoso, Andreas Wichert, Iterative random projections for
% high-dimensional data clustering, Pattern Recognition Letters, Volume 33,
% Issue 13, 1 October 2012, Pages 1749-1755, ISSN 0167-8655

function [ dimclust errorclust ] = dheur(X, k, dim, miss, sampling)
%Heuristic to estimate D from nearest neighbor errors
if (isempty(dim))
    %default dimensions to estimate nearest neighbor error
    dim = round(10.^[0:0.1:4]);
end

if (isempty(miss))
    %list of target nearest neighbor errors
    mh1=1/2;
    lambda = 1/8;
    l = 20;
    miss = mh1*exp(-lambda*(0:l-1));
end

if isempty(sampling)
    %percentage of data in X to use
    sampling = 1;
end

X = X(randsamplewr(size(X,1),floor(size(X,1)*sampling)),:);
[~,G] = min(sqeuclidean(X,X(randsamplewr(size(X,1),k),:)),[],2);
C = calccenters(X, G);


%estimate the nearest neighbor error for each dim
minimumMiss = miss(end);
for i=1:length(dim)
    proj = randn(size(X,2),dim(i));
    
    [~, nnOrig] =   min(sqeuclidean(X,C),[],2);
    [~,nnProj] = min(sqeuclidean(X*proj,C*proj),[],2);
    
    missDim(i) = mean(nnOrig~=nnProj);
    
    if ((missDim(i))<minimumMiss)
        dim = dim(1:i);
        break
    end
end

%select dims with nearest neighbor error according to miss
for i=1:length(miss)
    [~, idx] = min(abs(missDim-miss(i)));
    dimclust(i) = dim(idx);
    errorclust(i) = missDim(idx);
end

end

