% Written by ngelo Cardoso <angelo.cardoso@ist.utl.pt>
% 27 December 2012


function [G, iter]=simplekmeans(X,k,maxIter,singleton)
%K-means algorithm

%input
% X -data (samples x dimensions)

% k - number of clusters or initial centroids (number of clusters x dimensions)

% maxIter (optional) - maximum iterations for K-means (default: inf)

% singleton - replace emtpy clusters by furthest point from centroids
% (default: false)

%output
% G - cluster membership

% iter - number of iterations


%Parse arguments
error(nargchk(2,4, nargin));
if nargin<4
    singleton = [];
end
if nargin < 3
    maxIter = [];
end
if (isempty(maxIter))
    maxIter = inf;
end
if(isempty(singleton))
    singleton = false;
end

[n,m]=size(X);

if ~isscalar(k)
    C=k;
    k=size(C,1);
else
    C=X(randsample(n,k),:);
end

%initialize variables
gIter=nan(n,1);
G=zeros(n,1);
D=zeros(n,k);
iter = -1;
D = sqeuclidean(X,C);
[~,G]=min(D,[],2);

while any(gIter~=G) && iter<maxIter
    iter = iter + 1;
    gIter=G;
    
    %update cluster centers
    for t=1:k
        tIdx = G==t;
        if any(tIdx)
            C(t,:) = mean(X(tIdx,:));
        end
    end
    
    %find nearest center
    D = sqeuclidean(X,C);
    [minDist,G]=min(D,[],2);
    
    if (singleton)
        %assign empty clusters to furthest points from centroids
        while (length(unique(G))<k)
            [counter]=histc(G,1:k);
            emptyC = find(counter==0);
            candidates = find(ismember(G,find(counter>1)));
            [~,idxNewCentroids] = sort(minDist(candidates),'descend');
            G(candidates(idxNewCentroids(1:length(emptyC)))) = emptyC;
        end
        
    end
end

end